/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef PERCEPTION_IMPL_TYPE_POINTXYZIT_H
#define PERCEPTION_IMPL_TYPE_POINTXYZIT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint32.h"

namespace perception {
struct PointXYZIT {
    ::Float x{ 0.0F };
    ::Float y{ 0.0F };
    ::Float z{ 0.0F };
    ::UInt32 intensity{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(x);
        fun(y);
        fun(z);
        fun(intensity);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(x);
        fun(y);
        fun(z);
        fun(intensity);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("x", x);
        fun("y", y);
        fun("z", z);
        fun("intensity", intensity);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("x", x);
        fun("y", y);
        fun("z", z);
        fun("intensity", intensity);
    }

    friend bool operator==(const ::perception::PointXYZIT& lhs, const ::perception::PointXYZIT& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.x - rhs.x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.y - rhs.y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.z - rhs.z)) < DBL_EPSILON) && (lhs.intensity == rhs.intensity);
    }
};
} // namespace perception


#endif // PERCEPTION_IMPL_TYPE_POINTXYZIT_H
